﻿using System;
using System.Drawing;
using System.Windows.Forms;

using DevExpress.XtraEditors;
using DevExpress.XtraEditors.Controls;
using DevExpress.XtraTreeList;
using DevExpress.XtraTreeList.Columns;

namespace TestProject
{
    /// <summary>
    /// 메인 폼
    /// </summary>
    public partial class MainForm : XtraForm
    {
        //////////////////////////////////////////////////////////////////////////////////////////////////// Field
        ////////////////////////////////////////////////////////////////////////////////////////// Private

        #region Field

        /// <summary>
        /// 지역 컬럼
        /// </summary>
        private TreeListColumn regionColumn = null;

        /// <summary>
        /// 3월 판매액 컬럼
        /// </summary>
        private TreeListColumn marchSalesColumn = null;

        #endregion

        //////////////////////////////////////////////////////////////////////////////////////////////////// Constructor
        ////////////////////////////////////////////////////////////////////////////////////////// Public

        #region 생성자 - MainForm()

        /// <summary>
        /// 생성자
        /// </summary>
        public MainForm()
        {
            InitializeComponent();

            Load += Form_Load;
        }

        #endregion

        //////////////////////////////////////////////////////////////////////////////////////////////////// Method
        ////////////////////////////////////////////////////////////////////////////////////////// Private

        #region 폼 로드시 처리하기 - Form_Load(sender, e)

        /// <summary>
        /// 폼 로드시 처리하기
        /// </summary>
        /// <param name="sender">이벤트 발생자</param>
        /// <param name="e">이벤트 인자</param>
        private void Form_Load(object sender, EventArgs e)
        {
            this.treeList.RowHeight                              = 25;
            this.treeList.OptionsView.AutoWidth                  = false;
            this.treeList.OptionsBehavior.PopulateServiceColumns = true;
            this.treeList.OptionsBehavior.EditingMode            = TreeListEditingMode.EditForm;
            this.treeList.KeyFieldName                           = "ID";
            this.treeList.ParentFieldName                        = "RegionID";

            this.treeList.DataSource = SalesData.GetSalesDataList();

            this.regionColumn = this.treeList.Columns["Region"];

            this.regionColumn.OptionsColumn.ReadOnly = true;
            this.regionColumn.SortIndex              = 0;

            this.marchSalesColumn = this.treeList.Columns["MarchSales"];

            this.treeList.Columns[this.treeList.KeyFieldName   ].Visible = false;
            this.treeList.Columns[this.treeList.ParentFieldName].Visible = false;

            this.treeList.BestFitColumns();

            this.treeList.EditFormShowing  += treeList_EditFormShowing;
            this.treeList.EditFormPrepared += treeList_EditFormPrepared;
            this.treeList.EditFormHidden   += treeList_EditFormHidden;
        }

        #endregion
        #region 트리 리스트 편집 폼 표시시 처리하기 - treeList_EditFormShowing(sender, e)

        /// <summary>
        /// 트리 리스트 편집 폼 표시시 처리하기
        /// </summary>
        /// <param name="sender">이벤트 발생자</param>
        /// <param name="e">이벤트 인자</param>
        private void treeList_EditFormShowing(object sender, EditFormShowingEventArgs e)
        {
            if(e.Node.GetValue(this.regionColumn).Equals("Asia"))
            {
                e.Allow = false;
            }
        }

        #endregion
        #region 트리 리스트 편집 폼 준비시 처리하기 - treeList_EditFormPrepared(sender, e)

        /// <summary>
        /// 트리 리스트 편집 폼 준비시 처리하기
        /// </summary>
        /// <param name="sender">이벤트 발생자</param>
        /// <param name="e">이벤트 인자</param>
        private void treeList_EditFormPrepared(object sender, EditFormPreparedEventArgs e)
        {
            TreeList treeList = sender as TreeList;

            Control editor = e.BindableControls[treeList.FocusedColumn];

            if(editor != null)
            {
                editor.Focus();

                ((IContainerControl)e.Panel).ActivateControl(editor);
            }

            TextEdit textEdit = e.BindableControls[this.marchSalesColumn] as TextEdit;

            if(textEdit != null)
            {
                textEdit.Properties.Appearance.ForeColor = Color.Blue;

                textEdit.EditValueChanging += textEdit_EditValueChanging;
            }
        }

        #endregion
        #region 트리 리스트 편집 폼 숨김시 처리하기 - treeList_EditFormHidden(sender, e)

        /// <summary>
        /// 트리 리스트 편집 폼 숨김시 처리하기
        /// </summary>
        /// <param name="sender">이벤트 발생자</param>
        /// <param name="e">이벤트 인자</param>
        private void treeList_EditFormHidden(object sender, EditFormHiddenEventArgs e)
        {
            TextEdit textEdit = e.BindableControls[this.marchSalesColumn] as TextEdit;

            if(textEdit != null)
            {
                textEdit.EditValueChanging -= textEdit_EditValueChanging;
            }

            if(e.Result == EditFormResult.Update)
            {
                XtraMessageBox.Show("Changes are successfully saved.");
            }
        }

        #endregion
        #region 텍스트 에디터 편집 값 변경 전 처리하기 - textEdit_EditValueChanging(sender, e)

        /// <summary>
        /// 텍스트 에디터 편집 값 변경 전 처리하기
        /// </summary>
        /// <param name="sender">이벤트 발생자</param>
        /// <param name="e">이벤트 인자</param>
        private void textEdit_EditValueChanging(object sender, ChangingEventArgs e)
        {
        }

        #endregion
    }
}